@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: file_logger.h 4434 2008-10-14 08:52:13Z martin $
  */
 
 /**
@@ -59,4 +57,4 @@ struct file_logger_t {
  */
 file_logger_t *file_logger_create(FILE *out);
 
-#endif /* FILE_LOGGER_H_ @} */
+#endif /** FILE_LOGGER_H_ @}*/
