@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: sys_logger.h 4434 2008-10-14 08:52:13Z martin $
  */
 
 /**
@@ -61,4 +59,4 @@ struct sys_logger_t {
  */
 sys_logger_t *sys_logger_create(int facility);
 
-#endif /* SYS_LOGGER_H_ @} */
+#endif /** SYS_LOGGER_H_ @}*/
