@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: parser.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -78,4 +76,4 @@ struct parser_t {
  */
 parser_t *parser_create(chunk_t data);
 
-#endif /*PARSER_H_ @} */
+#endif /** PARSER_H_ @}*/
