@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: auth_payload.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -101,4 +99,4 @@ struct auth_payload_t {
  */
 auth_payload_t *auth_payload_create(void);
 
-#endif /* AUTH_PAYLOAD_H_ @} */
+#endif /** AUTH_PAYLOAD_H_ @}*/
