@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: configuration_attribute.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -53,7 +51,10 @@ enum configuration_attribute_type_t {
 	INTERNAL_IP6_DHCP = 12,
 	INTERNAL_IP4_SUBNET = 13,
 	SUPPORTED_ATTRIBUTES = 14,
-	INTERNAL_IP6_SUBNET = 15
+	INTERNAL_IP6_SUBNET = 15,
+	/* proprietary Microsoft attributes */
+	INTERNAL_IP4_SERVER = 23456,
+	INTERNAL_IP6_SERVER = 23457
 };
 
 /** 
@@ -124,4 +125,4 @@ struct configuration_attribute_t {
  */
 configuration_attribute_t *configuration_attribute_create(void);
 
-#endif /* CONFIGURATION_ATTRIBUTE_H_ @} */
+#endif /** CONFIGURATION_ATTRIBUTE_H_ @}*/
