@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: eap_payload.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -128,4 +126,4 @@ eap_payload_t *eap_payload_create_code(eap_code_t code, u_int8_t identifier);
  */
 eap_payload_t *eap_payload_create_nak(u_int8_t identifier);
 
-#endif /* EAP_PAYLOAD_H_ @} */
+#endif /** EAP_PAYLOAD_H_ @}*/
