@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: encodings.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -98,19 +96,6 @@ enum encoding_type_t {
 	 */
 	U_INT_32,
 	
-	/**
-	 * Representing a 64 Bit unsigned int value.
-	 * 
-	 * When generating it must be changed from host to network order.
-	 * The value is read from the associated data struct.
-	 * The current write position is moved 64 bit forward afterwards.
-	 * 
-	 * When parsing it must be changed from network to host order.
-	 * The value is written to the associated data struct.
-	 * The current read pointer is moved 64 bit forward afterwards.
-	 */
-	U_INT_64,
-	
 	/**
 	 * represents a RESERVED_BIT used in FLAG-Bytes.
 	 * 
@@ -527,4 +512,4 @@ struct encoding_rule_t {
 	u_int32_t offset;
 };
 
-#endif /*ENCODINGS_H_ @} */
+#endif /** ENCODINGS_H_ @}*/
