@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: encryption_payload.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -172,4 +170,4 @@ struct encryption_payload_t {
  */
 encryption_payload_t *encryption_payload_create(void);
 
-#endif /*ENCRYPTION_PAYLOAD_H_ @} */
+#endif /** ENCRYPTION_PAYLOAD_H_ @}*/
