@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: nonce_payload.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -32,7 +30,7 @@ typedef struct nonce_payload_t nonce_payload_t;
 /**
  * Nonce size in bytes for nonces sending to other peer.
  */
-#define NONCE_SIZE 16
+#define NONCE_SIZE 32
 
 /**
  * Length of a nonce payload without a nonce in bytes.
@@ -77,4 +75,4 @@ struct nonce_payload_t {
  */
 nonce_payload_t *nonce_payload_create(void);
 
-#endif /*NONCE_PAYLOAD_H_ @} */
+#endif /** NONCE_PAYLOAD_H_ @}*/
