@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: proposal_substructure.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -172,4 +170,4 @@ proposal_substructure_t *proposal_substructure_create(void);
 proposal_substructure_t *proposal_substructure_create_from_proposal(
 														proposal_t *proposal);
 
-#endif /*PROPOSAL_SUBSTRUCTURE_H_ @} */
+#endif /** PROPOSAL_SUBSTRUCTURE_H_ @}*/
