@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: unknown_payload.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -78,4 +76,4 @@ struct unknown_payload_t {
  */
 unknown_payload_t *unknown_payload_create(void);
 
-#endif /* UNKNOWN_PAYLOAD_H_ @} */
+#endif /** UNKNOWN_PAYLOAD_H_ @}*/
