@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: vendor_id_payload.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -86,4 +84,4 @@ struct vendor_id_payload_t {
  */
 vendor_id_payload_t *vendor_id_payload_create(void);
 
-#endif /* VENDOR_ID_PAYLOAD_H_ @} */
+#endif /** VENDOR_ID_PAYLOAD_H_ @}*/
