@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: packet.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -114,4 +112,4 @@ struct packet_t {
  */
 packet_t *packet_create(void);
 
-#endif /*PACKET_H_ @} */
+#endif /** PACKET_H_ @}*/
