@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: receiver.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -69,4 +67,4 @@ struct receiver_t {
  */
 receiver_t * receiver_create(void);
 
-#endif /*RECEIVER_H_ @} */
+#endif /** RECEIVER_H_ @}*/
