@@ -13,8 +13,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: socket.h 4647 2008-11-13 07:48:27Z martin $
  */
 
 /**
@@ -101,4 +99,4 @@ struct socket_t {
  */
 socket_t *socket_create();
 
-#endif /*SOCKET_H_ @} */
+#endif /** SOCKET_H_ @}*/
