@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: eap_aka_plugin.h 3491 2008-02-22 14:04:00Z martin $
  */
 
 /**
@@ -46,4 +44,4 @@ struct eap_aka_plugin_t {
  */
 plugin_t *plugin_create();
 
-#endif /* EAP_AKA_PLUGIN_H_ @}*/
+#endif /** EAP_AKA_PLUGIN_H_ @}*/
