@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: eap_sim_plugin.h 4406 2008-10-10 08:36:01Z martin $
  */
 
 /**
@@ -46,4 +44,4 @@ struct eap_sim_plugin_t {
  */
 plugin_t *plugin_create();
 
-#endif /* EAP_SIM_PLUGIN_H_ @}*/
+#endif /** EAP_SIM_PLUGIN_H_ @}*/
