@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: kernel_klips_plugin.h 4617 2008-11-11 08:45:19Z tobias $
  */
 
 /**
@@ -46,4 +44,4 @@ struct kernel_klips_plugin_t {
  */
 plugin_t *plugin_create();
 
-#endif /* KERNEL_KLIPS_PLUGIN_H_ @} */
+#endif /** KERNEL_KLIPS_PLUGIN_H_ @}*/
