@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: kernel_netlink_plugin.h 4358 2008-09-25 13:56:23Z tobias $
  */
 
 /**
@@ -46,4 +44,4 @@ struct kernel_netlink_plugin_t {
  */
 plugin_t *plugin_create();
 
-#endif /* KERNEL_NETLINK_PLUGIN_H_ @} */
+#endif /** KERNEL_NETLINK_PLUGIN_H_ @}*/
