@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: kernel_pfkey_plugin.h 4361 2008-10-01 16:47:51Z tobias $
  */
 
 /**
@@ -46,4 +44,4 @@ struct kernel_pfkey_plugin_t {
  */
 plugin_t *plugin_create();
 
-#endif /* KERNEL_PFKEY_PLUGIN_H_ @} */
+#endif /** KERNEL_PFKEY_PLUGIN_H_ @}*/
