@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: medsrv_plugin.h 3915 2008-05-08 12:11:30Z martin $
  */
 
 /**
@@ -46,4 +44,4 @@ struct medsrv_plugin_t {
  */
 plugin_t *plugin_create();
 
-#endif /* MEDSRV_PLUGIN_H_ @}*/
+#endif /** MEDSRV_PLUGIN_H_ @}*/
