@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: smp.h 3689 2008-03-28 12:44:01Z martin $
  */
 
 /**
@@ -49,4 +47,4 @@ struct smp_t {
  */
 plugin_t *plugin_create();
 
-#endif /* XML_H_ @}*/
+#endif /** XML_H_ @}*/
