@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: sql_plugin.h 3577 2008-03-12 14:38:35Z martin $
  */
 
 /**
@@ -46,4 +44,4 @@ struct sql_plugin_t {
  */
 plugin_t *plugin_create();
 
-#endif /* SQL_PLUGIN_H_ @}*/
+#endif /** SQL_PLUGIN_H_ @}*/
