@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: unit_tester.h 3491 2008-02-22 14:04:00Z martin $
  */
 
 /**
@@ -48,4 +46,4 @@ struct unit_tester_t {
  */
 plugin_t *plugin_create();
 
-#endif /* UNIT_TESTER_H_ @}*/
+#endif /** UNIT_TESTER_H_ @}*/
