@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: delete_ike_sa_job.c 3589 2008-03-13 14:14:44Z martin $
  */
 
 #include "delete_ike_sa_job.h"
@@ -63,6 +61,11 @@ static void execute(private_delete_ike_sa_job_t *this)
 											  this->ike_sa_id);
 	if (ike_sa)
 	{
+		if (ike_sa->get_state(ike_sa) == IKE_PASSIVE)
+		{
+			charon->ike_sa_manager->checkin(charon->ike_sa_manager, ike_sa);
+			return destroy(this);
+		}
 		if (this->delete_if_established)
 		{
 			if (ike_sa->delete(ike_sa) == DESTROY_ME)
