@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: job.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -51,4 +49,4 @@ struct job_t {
 	void (*destroy) (job_t *job);
 };
 
-#endif /* JOB_H_ @} */
+#endif /** JOB_H_ @}*/
