@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: rekey_child_sa_job.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -56,4 +54,4 @@ struct rekey_child_sa_job_t {
 rekey_child_sa_job_t *rekey_child_sa_job_create(u_int32_t reqid, 
 												protocol_id_t protocol,
 												u_int32_t spi);
-#endif /* REKEY_CHILD_SA_JOB_H_ @} */
+#endif /** REKEY_CHILD_SA_JOB_H_ @}*/
