@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: retransmit_job.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -54,4 +52,4 @@ struct retransmit_job_t {
 retransmit_job_t *retransmit_job_create(u_int32_t message_id,
 										ike_sa_id_t *ike_sa_id);
 
-#endif /* RETRANSMIT_JOB_H_ @} */
+#endif /** RETRANSMIT_JOB_H_ @}*/
