@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: roam_job.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -51,4 +49,4 @@ struct roam_job_t {
  */
 roam_job_t *roam_job_create(bool address);
 
-#endif /*ROAM_JOB_H_ @} */
+#endif /** ROAM_JOB_H_ @}*/
