@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: processor.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -93,4 +91,4 @@ struct processor_t {
  */
 processor_t *processor_create();
 
-#endif /*PROCESSOR_H_ @} */
+#endif /** PROCESSOR_H_ @}*/
