@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: ike_cert_post.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -52,4 +50,4 @@ struct ike_cert_post_t {
  */
 ike_cert_post_t *ike_cert_post_create(ike_sa_t *ike_sa, bool initiator);
 
-#endif /*  IKE_CERT_POST_H_ @} */
+#endif /** IKE_CERT_POST_H_ @}*/
