@@ -12,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: diffie_hellman.h 4566 2008-11-04 13:12:11Z martin $
  */
  
 /**
@@ -52,6 +50,8 @@ enum diffie_hellman_group_t {
 	ECP_521_BIT   = 21,
 	ECP_192_BIT   = 25,
 	ECP_224_BIT   = 26,
+	/** insecure NULL diffie hellman group for testing, in PRIVATE USE */
+	MODP_NULL = 1024,
 };
 
 /**
@@ -106,4 +106,4 @@ struct diffie_hellman_t {
 	void (*destroy) (diffie_hellman_t *this);
 };
 
-#endif /*DIFFIE_HELLMAN_H_ @} */
+#endif /** DIFFIE_HELLMAN_H_ @}*/
