@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: fips.h 3877 2008-04-26 09:40:22Z andreas $
  */
  
 /**
@@ -43,4 +41,4 @@ bool fips_compute_hmac_signature(const char *key, char *signature);
  */
 bool fips_verify_hmac_signature(const char *key, const char *signature);
 
-#endif /*FIPS_H_ @} */
+#endif /** FIPS_H_ @}*/
