@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: library.h 4311 2008-08-28 16:27:48Z martin $
  */
 
 /**
@@ -57,15 +55,15 @@
 #ifndef LIBRARY_H_
 #define LIBRARY_H_
 
+#include <printf_hook.h>
 #include <utils.h>
 #include <chunk.h>
 #include <settings.h>
-#include <printf_hook.h>
 #include <plugins/plugin_loader.h>
 #include <crypto/crypto_factory.h>
-#include <credentials/credential_factory.h>
 #include <fetcher/fetcher_manager.h>
 #include <database/database_factory.h>
+#include <credentials/credential_factory.h>
 
 typedef struct library_t library_t;
 
@@ -132,4 +130,4 @@ void library_deinit();
  */
 extern library_t *lib;
 
-#endif /* LIBRARY_H_ @}*/
+#endif /** LIBRARY_H_ @}*/
