@@ -47,4 +47,4 @@ struct aes_crypter_t {
 aes_crypter_t *aes_crypter_create(encryption_algorithm_t algo,
 								  size_t key_size);
 
-#endif /* AES_CRYPTER_H_ @}*/
+#endif /** AES_CRYPTER_H_ @}*/
