@@ -44,4 +44,4 @@ struct fips_prf_plugin_t {
  */
 plugin_t *plugin_create();
 
-#endif /* FIPS_PRF_PLUGIN_H_ @}*/
+#endif /** FIPS_PRF_PLUGIN_H_ @}*/
