@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: openssl_ec_public_key.h 4051 2008-06-10 09:08:27Z tobias $
  */
 
 /**
@@ -46,4 +44,4 @@ struct openssl_ec_public_key_t {
  */
 builder_t *openssl_ec_public_key_builder(key_type_t type);
 
-#endif /*OPENSSL_EC_PUBLIC_KEY_H_ @}*/
+#endif /** OPENSSL_EC_PUBLIC_KEY_H_ @}*/
