@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * $Id: openssl_hasher.h 4000 2008-05-22 12:13:10Z tobias $
  */
 
 /**
@@ -42,9 +40,8 @@ struct openssl_hasher_t {
  * Constructor to create openssl_hasher_t.
  * 
  * @param algo			algorithm
- * @param key_size		key size in bytes
  * @return				openssl_hasher_t, NULL if not supported
  */
 openssl_hasher_t *openssl_hasher_create(hash_algorithm_t algo);
 
-#endif /* OPENSSL_HASHER_H_ @}*/
+#endif /** OPENSSL_HASHER_H_ @}*/
