@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * $Id: openssl_plugin.h 4000 2008-05-22 12:13:10Z tobias $
  */
 
 /**
@@ -46,4 +44,4 @@ struct openssl_plugin_t {
  */
 plugin_t *plugin_create();
 
-#endif /* OPENSSL_PLUGIN_H_ @}*/
+#endif /** OPENSSL_PLUGIN_H_ @}*/
