@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: openssl_rsa_public_key.h 4000 2008-05-22 12:13:10Z tobias $
  */
 
 /**
@@ -46,4 +44,4 @@ struct openssl_rsa_public_key_t {
  */
 builder_t *openssl_rsa_public_key_builder(key_type_t type);
 
-#endif /*OPENSSL_RSA_PUBLIC_KEY_H_ @}*/
+#endif /** OPENSSL_RSA_PUBLIC_KEY_H_ @}*/
