@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: pubkey_public_key.h 3961 2008-05-15 12:33:00Z tobias $
  */
 
 /**
@@ -33,4 +31,4 @@
  */
 builder_t *pubkey_public_key_builder(key_type_t type);
 
-#endif /*PUBKEY_RSA_PUBLIC_KEY_H_ @}*/
+#endif /** PUBKEY_RSA_PUBLIC_KEY_H_ @}*/
