@@ -47,4 +47,4 @@ struct sha2_hasher_t {
  */
 sha2_hasher_t *sha2_hasher_create(hash_algorithm_t algorithm);
 
-#endif /* SHA2_HASHER_H_ @}*/
+#endif /** SHA2_HASHER_H_ @}*/
