@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: x509_cert.h 3650 2008-03-22 08:15:18Z andreas $
  */
 
 /**
@@ -46,4 +44,4 @@ struct x509_cert_t {
  */
 builder_t *x509_cert_builder(certificate_type_t type);
 
-#endif /* X509_CERT_H_ @}*/
+#endif /** X509_CERT_H_ @}*/
