@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id$
  */
 
 /**
@@ -25,7 +23,7 @@
 
 typedef struct settings_t settings_t;
 
-#include <library.h>
+#include <utils.h>
 #include <utils/enumerator.h>
 
 /**
@@ -116,4 +114,4 @@ struct settings_t {
  */
 settings_t *settings_create(char *file);
 
-#endif /* SETTINGS_H_ @}*/
+#endif /** SETTINGS_H_ @}*/
