@@ -1,4 +1,5 @@
 /*
+ * Copyright (C) 2008-2009 Tobias Brunner
  * Copyright (C) 2008 Martin Willi
  * Hochschule fuer Technik Rapperswil
  *
@@ -11,8 +12,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: utils.h 3820 2008-04-17 11:22:37Z martin $
  */
 
 /**
@@ -49,6 +48,11 @@
  */
 #define strneq(x,y,len) (strncmp(x, y, len) == 0)
 
+/**
+ * Macro compares two strings for equality ignoring case
+ */
+#define strcaseeq(x,y) (strcasecmp(x, y) == 0)
+
 /**
  * Macro compares two binary blobs for equality
  */
@@ -89,6 +93,16 @@
  */
 #define malloc_thing(thing) ((thing*)malloc(sizeof(thing)))
 
+/**
+ * Get the number of elements in an array
+ */
+#define countof(array) (sizeof(array)/sizeof(array[0]))
+
+/**
+ * Ignore result of functions tagged with warn_unused_result attributes
+ */
+#define ignore_result(call) { if(call); }
+
 /**
  * Assign a function as a class method
  */
@@ -102,9 +116,23 @@
 /**
  * General purpose boolean type.
  */
-typedef int bool;
-#define FALSE 0
-#define TRUE  1
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#else
+# ifndef HAVE__BOOL
+#  define _Bool signed char
+# endif /* HAVE__BOOL */
+# define bool _Bool
+# define false 0
+# define true 1
+# define __bool_true_false_are_defined 1
+#endif /* HAVE_STDBOOL_H */
+#ifndef FALSE
+# define FALSE false
+#endif /* FALSE */
+#ifndef TRUE
+# define TRUE  true
+#endif /* TRUE */
 
 typedef enum status_t status_t;
 
@@ -209,6 +237,21 @@ void *clalloc(void *pointer, size_t size);
  */
 void memxor(u_int8_t dest[], u_int8_t src[], size_t n);
 
+/**
+ * A variant of strstr with the characteristics of memchr, where haystack is not
+ * a null-terminated string but simply a memory area of length n.
+ */
+void *memstr(const void *haystack, const char *needle, size_t n);
+
+/**
+ * Creates a directory and all required parent directories. 
+ *
+ * @param	path	path to the new directory
+ * @param	mode	permissions of the new directory/directories 
+ * @return			TRUE on success
+ */
+bool mkdir_p(const char *path, mode_t mode);
+
 /**
  * returns null
  */
@@ -219,11 +262,29 @@ void *return_null();
  */
 void nop();
 
+/**
+ * returns TRUE
+ */
+bool return_true();
+
+/**
+ * returns FALSE
+ */
+bool return_false();
+
 /**
  * Special type to count references
  */
 typedef volatile u_int refcount_t;
 
+
+#ifdef HAVE_GCC_ATOMIC_OPERATIONS
+
+#define ref_get(ref) {__sync_fetch_and_add(ref, 1); }
+#define ref_put(ref) (!__sync_sub_and_fetch(ref, 1))
+
+#else /* !HAVE_GCC_ATOMIC_OPERATIONS */
+
 /**
  * Get a new reference.
  *
@@ -244,32 +305,33 @@ void ref_get(refcount_t *ref);
  */
 bool ref_put(refcount_t *ref);
 
+#endif /* HAVE_GCC_ATOMIC_OPERATIONS */
+
 /**
- * Get printf hooks for time.
+ * printf hook for time_t.
  *
  * Arguments are: 
- *    time_t* time
- * Arguments using #-specificer
  *    time_t* time, bool utc
  */
-printf_hook_functions_t time_get_printf_hooks();
+int time_printf_hook(char *dst, size_t len, printf_hook_spec_t *spec,
+					 const void *const *args);
 
 /**
- * Get printf hooks for time deltas.
+ * printf hook for time_t deltas.
  *
  * Arguments are: 
- *    time_t* delta
- * Arguments using #-specificer
  *    time_t* begin, time_t* end
  */
-printf_hook_functions_t time_delta_get_printf_hooks();
+int time_delta_printf_hook(char *dst, size_t len, printf_hook_spec_t *spec,
+						   const void *const *args);
 
 /**
- * Get printf hooks for time deltas.
+ * printf hook for memory areas.
  *
  * Arguments are: 
  *    u_char *ptr, int len
  */
-printf_hook_functions_t mem_get_printf_hooks();
+int mem_printf_hook(char *dst, size_t len, printf_hook_spec_t *spec,
+					const void *const *args);
 
-#endif /* UTILS_H_ @}*/
+#endif /** UTILS_H_ @}*/
