@@ -11,10 +11,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: leak_detective.c 4609 2008-11-10 16:44:27Z martin $
  */
 	
+#define _GNU_SOURCE
+#include <sched.h>
 #include <stddef.h>
 #include <string.h>
 #include <stdio.h>
@@ -27,7 +27,6 @@
 #include <syslog.h>
 #include <pthread.h>
 #include <netdb.h>
-#include <printf.h>
 #include <locale.h>
 
 #include "leak_detective.h"
@@ -160,6 +159,7 @@ char *whitelist[] = {
 	/* pthread stuff */
 	"pthread_create",
 	"pthread_setspecific",
+	"__pthread_setspecific",
 	/* glibc functions */
 	"mktime",
 	"__gmtime_r",
@@ -199,6 +199,12 @@ char *whitelist[] = {
 	"RSA_new_method",
 	"DH_new_method",
 	"ENGINE_load_builtin_engines",
+	"OPENSSL_config",
+	/* libgcrypt */
+	"gcry_control",
+	"gcry_check_version",
+	"gcry_randomize",
+	"gcry_create_nonce",
 };
 
 /**
@@ -471,6 +477,16 @@ leak_detective_t *leak_detective_create()
 	
 	if (getenv("LEAK_DETECTIVE_DISABLE") == NULL)
 	{
+		cpu_set_t mask;
+		
+		CPU_ZERO(&mask);
+		CPU_SET(0, &mask);
+		
+		if (sched_setaffinity(0, sizeof(cpu_set_t), &mask) != 0)
+		{
+			fprintf(stderr, "setting CPU affinity failed: %m");
+		}
+	
 		lib->leak_detective = TRUE;
 		install_hooks();
 	}
