@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: main.c 3967 2008-05-16 08:52:32Z martin $
  */
 
 #include <dispatcher.h>
@@ -42,7 +40,7 @@ int main (int arc, char *argv[])
 	
 	socket = lib->settings->get_str(lib->settings, "manager.socket", NULL);
 	debug = lib->settings->get_bool(lib->settings, "manager.debug", FALSE);
-	timeout = lib->settings->get_int(lib->settings, "manager.timeout", 900);
+	timeout = lib->settings->get_time(lib->settings, "manager.timeout", 900);
 	threads = lib->settings->get_int(lib->settings, "manager.threads", 10);
 	database = lib->settings->get_str(lib->settings, "manager.database", NULL);
 	if (!database)
