@@ -11,8 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- *
- * $Id: manager.h 3589 2008-03-13 14:14:44Z martin $
  */
 
 /**
@@ -95,4 +93,4 @@ struct manager_t {
  */
 manager_t *manager_create(storage_t *storage);
 
-#endif /* MANAGER_H_ @} */
+#endif /** MANAGER_H_ @}*/
